"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[7806],{7806(e){e.exports=JSON.parse('{"grafana-sql":{"components":{"confirm-modal":{"builder-mode":"O modo de construtor n\xE3o exibe as altera\xE7\xF5es feitas no c\xF3digo. O construtor de consultas exibir\xE1 as \xFAltimas altera\xE7\xF5es que efetuou no modo de construtor.","cancel":"Cancelar","clipboard":"Pretende copiar o seu c\xF3digo para a \xE1rea de transfer\xEAncia?","copy-code-and-switch":"Copiar c\xF3digo e mudar","discard-code-and-switch":"Descartar c\xF3digo e mudar","warning":"Aviso"},"connection-limits":{"auto-max-idle":"Ralenti m\xE1ximo autom\xE1tico","content-auto-max-idle":"","content-max-idle":"","content-max-lifetime":"A quantidade m\xE1xima de tempo em segundos que uma liga\xE7\xE3o pode ser reutilizada. Se definido para 0, as liga\xE7\xF5es ser\xE3o reutilizadas para sempre.","content-max-open":"","max-idle":"Ralenti m\xE1x.","max-lifetime":"Tempo de vida m\xE1ximo","max-open":"Aberturas m\xE1x.","title-connection-limits":"Limites da liga\xE7\xE3o"},"dataset-selector":{"aria-label-dataset-selector":"Seletor de conjunto de dados"},"error-boundary":{"fall-back":{"error":"Erro"}},"get-custom-operators":{"custom-operators":{"label":{"macros":"Macros"}}},"make-render-column":{"render-column":{"aria-label-group-by":"Agrupar por","title-remove-group-by-column":"Remover grupo por coluna"}},"order-by-row":{"aria-label-order-by":"Ordenar por","label-limit":"Limite","label-offset":"Deslocamento","label-order-by":"Ordenar por"},"preview":{"label-element":{"preview":"Pr\xE9-visualizar","tooltip-copy-to-clipboard":"Copiar para a \xE1rea de transfer\xEAncia"}},"query-header":{"content-invalid-query":"","editor-modes":{"label-builder":"Construtor","label-code":"C\xF3d."},"label-dataset":"Conjunto de dados","label-filter":"Filtro","label-format":"Formato","label-group":"Grupo","label-order":"Encomenda","label-preview":"Pr\xE9-visualizar","label-table":"Tabela","placeholder-select-format":"Selecionar formato","run-query":"Executar consulta"},"query-meta":{"variables":{"textField":"","valueField":""}},"query-toolbox":{"content-hit-ctrlcmdreturn-to-run-query":"Pressione CTRL/CMD+Return para executar a consulta","tooltip-collapse":"Recolher editor","tooltip-expand":"Expandir editor","tooltip-format-query":"Formatar consulta"},"query-validator":{"query-will-process":"","validating-query":"A validar consulta..."},"raw-editor":{"render-placeholder":{"editing-in-expanded-code-editor":"Editar no editor de c\xF3digo expandido"},"title-query-num":"Consulta {{queryNum}}"},"select-column":{"label-column":"Coluna"},"select-custom-function-parameters":{"aria-label-parameter":"Par\xE2metro {{index}} para coluna {{columnIndex}}","render-parameters":{"params":{"title-remove-parameter":"Remover par\xE2metro"}},"title-add-parameter":"Adicionar par\xE2metro"},"select-row":{"aggregate-options":{"options":{"label":{"aggregations":"Agrega\xE7\xF5es","macros":"Macros"}}},"label":{"time":"do fumo","value":"valor"},"label-alias":"Nome alternativo","label-data-operations":"Opera\xE7\xF5es de dados","title-add-column":"Adicionar coluna","title-remove-column":"Remover coluna"},"settings":{"aria-label-conjunction":"Conjun\xE7\xE3o","aria-label-field":"Campo","aria-label-operator":"Operador","title-button-filter":"Filtro {{ buttonLabel }}"},"sql-query-editor-lazy":{"text-loading-editor":"A carregar o editor..."},"table-selector":{"aria-label-table-selector":"Seletor de tabela","placeholder-loading":"A carregar tabelas","placeholder-select-table":"Selecionar tabela"},"tlssecrets-config":{"content-tlsssl-client-certificate":"Para autenticar com um certificado de cliente TLS/SSL, forne\xE7a o certificado de cliente aqui.","content-tlsssl-client-key":"Para autenticar com um certificado TLS/SSL de cliente, forne\xE7a a chave aqui.","content-tlsssl-root-certificate":"Se o modo TLS/SSL selecionado exigir um certificado de raiz do servidor, forne\xE7a-o aqui","tlsssl-client-certificate":"Certificado de cliente TLS/SSL","tlsssl-client-key":"Chave de cliente TLS/SSL","tlsssl-root-certificate":"Certificado de raiz TLS/SSL"},"visual-editor":{"label-filter-by-column-value":"Filtrar por valor de coluna","label-group-by-column":"Agrupar por coluna"},"widgets":{"aria-label-macros-value-selector":"Seletor de valor de macros"}},"utils":{"get-columns-width-indices":{"label-selected-columns":"Colunas selecionadas"}}}}')}}]);

//# sourceMappingURL=7806.648065afd5728eccab2c.js.map