"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[3630],{71249(e){e.exports=JSON.parse('{"grafana-sql":{"components":{"confirm-modal":{"builder-mode":"Olu\u015Fturucu modu, kodda yap\u0131lan de\u011Fi\u015Fiklikleri g\xF6r\xFCnt\xFClemez. Sorgu olu\u015Fturucu, olu\u015Fturucu modunda yapt\u0131\u011F\u0131n\u0131z son de\u011Fi\u015Fiklikleri g\xF6r\xFCnt\xFCler.","cancel":"\u0130ptal et","clipboard":"Kodunuzu panoya kopyalamak istiyor musunuz?","copy-code-and-switch":"Kodu kopyala ve ge\xE7i\u015F yap","discard-code-and-switch":"Kodu sil ve ge\xE7i\u015F yap","warning":"Uyar\u0131"},"connection-limits":{"auto-max-idle":"Otomatik maksimum bo\u015Fta ba\u011Flant\u0131 say\u0131s\u0131","content-auto-max-idle":"","content-max-idle":"","content-max-lifetime":"Saniye cinsinden bir ba\u011Flant\u0131n\u0131n yeniden kullan\u0131labilece\u011Fi maksimum s\xFCre. 0 olarak ayarlan\u0131rsa ba\u011Flant\u0131lar s\xFCresiz olarak yeniden kullan\u0131l\u0131r.","content-max-open":"","max-idle":"Maks. bo\u015Fta","max-lifetime":"Maksimum kullan\u0131m s\xFCresi","max-open":"Maks. a\xE7\u0131k","title-connection-limits":"Ba\u011Flant\u0131 s\u0131n\u0131rlar\u0131"},"dataset-selector":{"aria-label-dataset-selector":"Veri k\xFCmesi se\xE7ici"},"error-boundary":{"fall-back":{"error":"Hata"}},"get-custom-operators":{"custom-operators":{"label":{"macros":"Makrolar"}}},"make-render-column":{"render-column":{"aria-label-group-by":"Gruplama \xF6l\xE7\xFCt\xFC","title-remove-group-by-column":"Grubu s\xFCtuna g\xF6re kald\u0131r"}},"order-by-row":{"aria-label-order-by":"S\u0131ralama \xF6l\xE7\xFCt\xFC","label-limit":"S\u0131n\u0131rlama","label-offset":"Ofset","label-order-by":"S\u0131ralama \xF6l\xE7\xFCt\xFC"},"preview":{"label-element":{"preview":"\xD6n izleme","tooltip-copy-to-clipboard":"Panoya kopyala"}},"query-header":{"content-invalid-query":"","editor-modes":{"label-builder":"Olu\u015Fturucu","label-code":"Kod"},"label-dataset":"Veri k\xFCmesi","label-filter":"Filtreleyin","label-format":"Bi\xE7im","label-group":"Grup","label-order":"Sipari\u015F","label-preview":"\xD6n izleme","label-table":"Tablo","placeholder-select-format":"Bi\xE7imi se\xE7in","run-query":"Sorgu \xE7al\u0131\u015Ft\u0131r"},"query-meta":{"variables":{"textField":"","valueField":""}},"query-toolbox":{"content-hit-ctrlcmdreturn-to-run-query":"Sorguyu \xE7al\u0131\u015Ft\u0131rmak i\xE7in CTRL/CMD+Return tu\u015Flar\u0131na bas\u0131n","tooltip-collapse":"D\xFCzenleyiciyi daralt","tooltip-expand":"D\xFCzenleyiciyi geni\u015Flet","tooltip-format-query":"Sorguyu bi\xE7imlendir"},"query-validator":{"query-will-process":"","validating-query":"Sorgu do\u011Frulan\u0131yor..."},"raw-editor":{"render-placeholder":{"editing-in-expanded-code-editor":"Geni\u015Fletilmi\u015F kod d\xFCzenleyicide d\xFCzenleme"},"title-query-num":"{{queryNum}}. sorgu"},"select-column":{"label-column":"S\xFCtun"},"select-custom-function-parameters":{"aria-label-parameter":"{{columnIndex}} s\xFCtunu i\xE7in {{index}} parametresi ","render-parameters":{"params":{"title-remove-parameter":"Parametreyi kald\u0131r"}},"title-add-parameter":"Parametre ekle"},"select-row":{"aggregate-options":{"options":{"label":{"aggregations":"Toplamalar","macros":"Makrolar"}}},"label":{"time":"zaman","value":"de\u011Fer"},"label-alias":"Takma isim","label-data-operations":"Veri i\u015Flemleri","title-add-column":"S\xFCtun ekle","title-remove-column":"S\xFCtunu kald\u0131r"},"settings":{"aria-label-conjunction":"Birle\u015Fim","aria-label-field":"Alan","aria-label-operator":"\u0130\u015Fle\xE7","title-button-filter":"{{ buttonLabel }} filtresi"},"sql-query-editor-lazy":{"text-loading-editor":"D\xFCzenleyici y\xFCkleniyor"},"table-selector":{"aria-label-table-selector":"Tablo se\xE7ici","placeholder-loading":"Tablolar y\xFCkleniyor","placeholder-select-table":"Tablo se\xE7in"},"tlssecrets-config":{"content-tlsssl-client-certificate":"Bir TLS/SSL istemci sertifikas\u0131yla kimlik do\u011Frulamak i\xE7in istemci sertifikas\u0131n\u0131 burada sa\u011Flay\u0131n.","content-tlsssl-client-key":"Bir istemci TLS/SSL sertifikas\u0131yla kimlik do\u011Frulamak i\xE7in anahtar\u0131 burada sa\u011Flay\u0131n.","content-tlsssl-root-certificate":"Se\xE7ilen TLS/SSL modu bir sunucu k\xF6k sertifikas\u0131 gerektiriyorsa onu da burada sa\u011Flay\u0131n","tlsssl-client-certificate":"TLS/SSL \u0130stemci Sertifikas\u0131","tlsssl-client-key":"TLS/SSL \u0130stemci Anahtar\u0131","tlsssl-root-certificate":"TLS/SSL K\xF6k Sertifikas\u0131"},"visual-editor":{"label-filter-by-column-value":"S\xFCtun de\u011Ferine g\xF6re filtreleyin","label-group-by-column":"S\xFCtuna g\xF6re gruplay\u0131n"},"widgets":{"aria-label-macros-value-selector":"Makro de\u011Fer se\xE7ici"}},"utils":{"get-columns-width-indices":{"label-selected-columns":"Se\xE7ilen s\xFCtunlar"}}}}')}}]);

//# sourceMappingURL=3630.425334db172c46c54cd1.js.map